from django.conf import settings
from django.core.mail import send_mail
from django.http import JsonResponse
from django.template import loader
from django.utils.translation import gettext_lazy as _
from djangoldp.filters import LocalObjectOnContainerPathBackend
from djangoldp.models import Model
from djangoldp.views import LDPViewSet
from djangoldp_application.models import (
    Application,
    ApplicationComponent,
    ApplicationComponentParameter,
    ApplicationGraphics,
    ApplicationNPM,
    ApplicationPackage,
    ApplicationPackageParameter,
    ApplicationService,
    ComponentExtension,
    ComponentExtensionParameter,
    Federation,
)
from djangoldp_component.models import Component, Package
from rest_framework import response, serializers, status, viewsets
from rest_framework_yaml.parsers import YAMLParser
from rest_framework_yaml.renderers import YAMLRenderer

EMAIL_FROM = getattr(settings, "DEFAULT_FROM_EMAIL", False) or getattr(
    settings, "EMAIL_HOST_USER", False
)
ANSIBLE_SERVERS = set({"127.0.0.1"})


if hasattr(settings, "ANSIBLE_SERVERS"):
    ANSIBLE_SERVERS = ANSIBLE_SERVERS.union(getattr(settings, "ANSIBLE_SERVERS"))


def get_client_ip(request):
    x_forwarded_for = request.headers.get("x-forwarded-for")
    if x_forwarded_for:
        ip = x_forwarded_for.split(",")[0]
    else:
        ip = request.META.get("REMOTE_ADDR")
    return ip


def format(value):
    if not value == str(value):
        return ""
    v = value.lower()
    if v == "false":
        return False
    elif v == "true":
        return True
    else:
        return value.replace("\r\n", "\n").replace("\n\n", "\n")


class ApplicationSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Application
        fields = ("slug", "deploy")


class ApplicationSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Application
        fields = ("slug", "deploy")


class ApplicationDetailSerializer(serializers.HyperlinkedModelSerializer):
    def to_representation(self, obj):
        application = super().to_representation(obj)

        federation = []
        for host in application["federation"]:
            federation.append(Federation.objects.get(urlid=host).target.api_url)

        serialized = {"apps": {"hosts": {}}}
        serialized["apps"]["hosts"][application["slug"]] = {
            "graphics": {},
            "data": {"api": application["api_url"], "with": federation},
            "packages": [],
            "components": [],
        }

        if application["client_url"]:
            serialized["apps"]["hosts"][application["slug"]]["graphics"][
                "client"
            ] = application["client_url"]

        if application["repository"]:
            serialized["apps"]["hosts"][application["slug"]]["graphics"][
                "canva"
            ] = application["repository"]

        if application["application_title"]:
            serialized["apps"]["hosts"][application["slug"]]["graphics"][
                "title"
            ] = application["application_title"]

        if application["application_logo"]:
            serialized["apps"]["hosts"][application["slug"]]["graphics"][
                "logo"
            ] = application["application_logo"]

        if len(application["graphics"]) > 0:
            for applicationGraphic in application["graphics"]:
                if applicationGraphic.obj.primary_key:
                    if (
                        applicationGraphic.obj.primary_key
                        not in serialized["apps"]["hosts"][application["slug"]][
                            "graphics"
                        ]
                    ):
                        serialized["apps"]["hosts"][application["slug"]]["graphics"][
                            applicationGraphic.obj.primary_key
                        ] = {}
                    serialized["apps"]["hosts"][application["slug"]]["graphics"][
                        applicationGraphic.obj.primary_key
                    ][applicationGraphic.obj.key] = format(applicationGraphic.obj.value)
                else:
                    serialized["apps"]["hosts"][application["slug"]]["graphics"][
                        applicationGraphic.obj.key
                    ] = format(applicationGraphic.obj.value)

        if len(application["services"]) > 0:
            serialized["apps"]["hosts"][application["slug"]]["services"] = {}
            for applicationService in application["services"]:
                if applicationService.obj.primary_key:
                    if (
                        applicationService.obj.primary_key
                        not in serialized["apps"]["hosts"][application["slug"]][
                            "services"
                        ]
                    ):
                        serialized["apps"]["hosts"][application["slug"]]["services"][
                            applicationService.obj.primary_key
                        ] = {}
                    serialized["apps"]["hosts"][application["slug"]]["services"][
                        applicationService.obj.primary_key
                    ][applicationService.obj.key] = format(applicationService.obj.value)
                else:
                    serialized["apps"]["hosts"][application["slug"]]["services"][
                        applicationService.obj.key
                    ] = format(applicationService.obj.value)

        if len(application["npms"]) > 0:
            serialized["apps"]["hosts"][application["slug"]]["npm"] = []
            for applicationNPM in application["npms"]:
                serialized["apps"]["hosts"][application["slug"]]["npm"].append(
                    {
                        "package": applicationNPM.obj.package,
                        "version": applicationNPM.obj.version,
                    }
                )

        for applicationComponent in application["components"]:
            component = Component.objects.get(id=applicationComponent.obj.component_id)
            insertComponent = {
                "type": component.name,
                "route": format(component.preferred_route),
                "parameters": {},
                "extensions": [],
                "experimental": [],
            }
            if component.auto_import:
                insertComponent["experimental"].append("routing")
            if component.auto_menu:
                insertComponent["experimental"].append("menu")
            keys = []
            for parameter in applicationComponent.obj.parameters.all():
                if parameter.key == "route":
                    insertComponent["route"] = format(parameter.value)
                    keys.append(parameter.key)
                elif parameter.key == "defaultRoute":
                    insertComponent["defaultRoute"] = format(parameter.value)
                    keys.append(parameter.key)
                elif not parameter.key in keys:
                    insertComponent["parameters"][parameter.key] = format(
                        parameter.value
                    )
                    keys.append(parameter.key)
                elif hasattr(
                    insertComponent["parameters"][parameter.key], "__len__"
                ) and (
                    not isinstance(insertComponent["parameters"][parameter.key], str)
                ):
                    insertComponent["parameters"][parameter.key].append(
                        format(parameter.value)
                    )
                else:
                    insertComponent["parameters"][parameter.key] = [
                        insertComponent["parameters"][parameter.key],
                        format(parameter.value),
                    ]
            missingKeys = []
            for parameter in component.parameters.all():
                if not parameter.key in keys:
                    if parameter.key == "route":
                        insertComponent["route"] = format(parameter.default)
                        missingKeys.append(parameter.key)
                    elif parameter.key == "defaultRoute":
                        insertComponent["defaultRoute"] = format(parameter.default)
                        keys.append(parameter.key)
                    elif not parameter.key in missingKeys:
                        insertComponent["parameters"][parameter.key] = format(
                            parameter.default
                        )
                        missingKeys.append(parameter.key)
                    elif hasattr(
                        insertComponent["parameters"][parameter.key], "__len__"
                    ) and (
                        not isinstance(
                            insertComponent["parameters"][parameter.key], str
                        )
                    ):
                        insertComponent["parameters"][parameter.key].append(
                            format(parameter.default)
                        )
                    else:
                        insertComponent["parameters"][parameter.key] = [
                            insertComponent["parameters"][parameter.key],
                            format(parameter.default),
                        ]
            for extensionComponent in applicationComponent.obj.extensions.all():
                extension = Component.objects.get(id=extensionComponent.component_id)
                componentExtension = {
                    "type": extension.name,
                    "route": format(component.preferred_route),
                    "parameters": {},
                    "experimental": [],
                }
                if component.auto_import:
                    insertComponent["experimental"].append("routing")
                if component.auto_menu:
                    insertComponent["experimental"].append("menu")
                keys = []
                for parameter in extensionComponent.parameters.all():
                    if parameter.key == "route":
                        componentExtension["route"] = format(parameter.value)
                        keys.append(parameter.key)
                    elif parameter.key == "defaultRoute":
                        componentExtension["defaultRoute"] = format(parameter.value)
                        keys.append(parameter.key)
                    elif not parameter.key in keys:
                        componentExtension["parameters"][parameter.key] = format(
                            parameter.value
                        )
                        keys.append(parameter.key)
                    elif hasattr(
                        insertComponent["parameters"][parameter.key], "__len__"
                    ) and (
                        not isinstance(
                            componentExtension["parameters"][parameter.key], str
                        )
                    ):
                        componentExtension["parameters"][parameter.key].append(
                            format(parameter.value)
                        )
                    else:
                        componentExtension["parameters"][parameter.key] = [
                            componentExtension["parameters"][parameter.key],
                            format(parameter.value),
                        ]
                missingKeys = []
                for parameter in extension.parameters.all():
                    if not parameter.key in keys:
                        if parameter.key == "route":
                            componentExtension["route"] = format(parameter.default)
                            missingKeys.append(parameter.key)
                        elif parameter.key == "defaultRoute":
                            componentExtension["defaultRoute"] = format(
                                parameter.default
                            )
                            missingKeys.append(parameter.key)
                        elif not parameter.key in missingKeys:
                            componentExtension["parameters"][parameter.key] = format(
                                parameter.default
                            )
                            missingKeys.append(parameter.key)
                        elif hasattr(
                            componentExtension["parameters"][parameter.key], "__len__"
                        ) and (
                            not isinstance(
                                componentExtension["parameters"][parameter.key], str
                            )
                        ):
                            componentExtension["parameters"][parameter.key].append(
                                format(parameter.default)
                            )
                        else:
                            componentExtension["parameters"][parameter.key] = [
                                componentExtension["parameters"][parameter.key],
                                format(parameter.default),
                            ]
                insertComponent["extensions"].append(componentExtension)
            serialized["apps"]["hosts"][application["slug"]]["components"].append(
                insertComponent
            )

        insertDependencies = []
        for applicationPackage in application["packages"]:
            package = Package.objects.get(id=applicationPackage.obj.package_id)
            insertDependency = {
                "distribution": package.distribution,
                "module": package.module,
                "parameters": {},
            }
            keys = []
            for parameter in applicationPackage.obj.parameters.all():
                if not parameter.key in keys:
                    insertDependency["parameters"][parameter.key] = format(
                        parameter.value
                    )
                    keys.append(parameter.key)
                elif hasattr(
                    insertDependency["parameters"][parameter.key], "__len__"
                ) and (
                    not isinstance(insertDependency["parameters"][parameter.key], str)
                ):
                    insertDependency["parameters"][parameter.key].append(
                        format(parameter.value)
                    )
                else:
                    insertDependency["parameters"][parameter.key] = [
                        insertDependency["parameters"][parameter.key],
                        format(parameter.value),
                    ]
            missingKeys = []
            for parameter in package.parameters.all():
                if not parameter.key in keys:
                    if not parameter.key in missingKeys:
                        insertDependency["parameters"][parameter.key] = format(
                            parameter.default
                        )
                        missingKeys.append(parameter.key)
                    elif hasattr(
                        insertDependency["parameters"][parameter.key], "__len__"
                    ) and (
                        not isinstance(
                            insertDependency["parameters"][parameter.key], str
                        )
                    ):
                        insertDependency["parameters"][parameter.key].append(
                            format(parameter.default)
                        )
                    else:
                        insertDependency["parameters"][parameter.key] = [
                            insertDependency["parameters"][parameter.key],
                            format(parameter.default),
                        ]
            insertDependencies.append(insertDependency)
        serialized["apps"]["hosts"][application["slug"]][
            "packages"
        ] = insertDependencies

        return serialized

    class Meta:
        model = Application
        lookup_field = "slug"
        fields = [
            "urlid",
            "slug",
            "api_url",
            "client_url",
            "application_title",
            "application_logo",
            "services",
            "graphics",
            "npms",
            "components",
            "packages",
            "repository",
            "federation",
        ]
        extra_kwargs = {"url": {"lookup_field": "slug"}}


class ApplicationViewSet(viewsets.ModelViewSet):
    queryset = Application.objects.all()
    serializer_class = ApplicationSerializer
    parser_classes = (YAMLParser,)
    renderer_classes = (YAMLRenderer,)


class ApplicationDetailViewSet(viewsets.ModelViewSet):
    queryset = Application.objects.all()
    serializer_class = ApplicationDetailSerializer
    lookup_field = "slug"
    parser_classes = (YAMLParser,)
    renderer_classes = (YAMLRenderer,)


class CreateApplicationViewSet(LDPViewSet):
    model = Application
    depth = 0

    def __init__(self, **kwargs):
        viewsets.ModelViewSet.__init__(self, **kwargs)
        if self.permission_classes:
            filtered_classes = [
                p
                for p in self.permission_classes
                if hasattr(p, "filter_backends") and p.filter_backends is not None
            ]
            for p in filtered_classes:
                self.filter_backends = list(
                    set(self.filter_backends).union(set(p.filter_backends))
                )

        if getattr(settings, "DISABLE_LOCAL_OBJECT_FILTER", False):
            if LocalObjectOnContainerPathBackend in self.filter_backends:
                self.filter_backends.remove(LocalObjectOnContainerPathBackend)

        self.serializer_class = self.build_read_serializer()
        self.write_serializer_class = self.build_write_serializer()

    def create(self, request, *args, **kwargs):
        self.check_model_permissions(request)
        serializer = self.get_write_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        if not self.is_safe_create(request.user, serializer.validated_data):
            return response.Response(
                {"detail": "You do not have permission to perform this action"},
                status=status.HTTP_403_FORBIDDEN,
            )

        entry = self.perform_create(serializer)

        if serializer.initial_data["source"]:
            container, source_application = Model.resolve(
                serializer.initial_data["source"]["@id"]
            )

            try:
                for component_relation in source_application.components.all():
                    (
                        new_component_relation,
                        created,
                    ) = ApplicationComponent.objects.get_or_create(
                        component=component_relation.component, application=entry
                    )
                    for parameter in component_relation.parameters.all():
                        ApplicationComponentParameter.objects.create(
                            component=new_component_relation,
                            key=parameter.key,
                            value=parameter.value,
                        )
                    for extension in component_relation.extensions.all():
                        (
                            new_extension_relation,
                            created,
                        ) = ComponentExtension.objects.get_or_create(
                            base_component=new_component_relation,
                            component=extension.component,
                        )
                        for parameter in new_extension_relation.parameters.all():
                            ComponentExtensionParameter.objects.create(
                                component=new_extension_relation,
                                key=parameter.key,
                                value=parameter.value,
                            )
            except:
                pass

            try:
                for package_relation in source_application.packages.all():
                    (
                        new_package_relation,
                        created,
                    ) = ApplicationPackage.objects.get_or_create(
                        package=package_relation.package, application=entry
                    )
                    for parameter in package_relation.parameters.all():
                        ApplicationPackageParameter.objects.create(
                            package=new_package_relation,
                            key=parameter.key,
                            value=parameter.value,
                        )
            except:
                pass

            try:
                for application_npm in source_application.npms.all():
                    ApplicationNPM.objects.get_or_create(
                        application=entry,
                        package=application_npm.package,
                        version=application_npm.version,
                    )
            except:
                pass

            try:
                for application_service in source_application.services.all():
                    ApplicationService.objects.get_or_create(
                        application=entry,
                        primary_key=application_service.primary_key,
                        key=application_service.key,
                        value=application_service.value,
                    )
            except:
                pass

            try:
                for application_graphic in source_application.graphics.all():
                    ApplicationGraphics.objects.get_or_create(
                        application=entry,
                        primary_key=application_graphic.primary_key,
                        key=application_graphic.key,
                        value=application_graphic.value,
                    )
            except:
                pass

            try:
                for application_federation in source_application.federation.all():
                    Federation.objects.get_or_create(
                        application=entry, target=application_federation.target
                    )
            except:
                pass

        response_serializer = self.get_serializer()
        container, updated_entry = Model.resolve(serializer.instance.urlid)
        data = response_serializer.to_representation(updated_entry)
        headers = self.get_success_headers(data)
        return response.Response(data, status=status.HTTP_201_CREATED, headers=headers)


def mark_as_doing(request, slug):
    if request.method == "GET" and get_client_ip(request) in ANSIBLE_SERVERS:
        application = Application.objects.get(slug=slug)
        for deploy in application.deployments.filter(status="Todo"):
            deploy.status = "Doing"
            deploy.save()
        return JsonResponse({"status": "success"}, status=200)
    else:
        return JsonResponse({"status": "invalid request"}, status=400)


def mark_as_done(request, slug):
    if request.method == "GET" and get_client_ip(request) in ANSIBLE_SERVERS:
        application = Application.objects.get(slug=slug)
        for deploy in application.deployments.filter(status="Doing"):
            deploy.status = "Done"
            deploy.save()

            if deploy.requester:
                requester = deploy.requester
            else:
                requester = application.creator

            if application.deployments.count() == 1:
                html_message = loader.render_to_string(
                    "djangoldp_application/email.html",
                    {
                        "message": _("Deployment done. You can use the default account: admin/admin. Don't forget to create your own account."),
                        "link": application.client_url,
                        "object": "{} https://{}".format(
                            _("About your deployment of"), application.client_url
                        ),
                    },
                )
            else :
                html_message = loader.render_to_string(
                    "djangoldp_application/email.html",
                    {
                        "message": _("Deployment done."),
                        "link": application.client_url,
                        "object": "{} https://{}".format(
                            _("About your deployment of"), application.client_url
                        ),
                    },
                )

            send_mail(
                "{} (https://{})".format(
                    _("Déploiement de ") + application.application_title,
                    application.client_url,
                ),
                _("Deployment done"),
                EMAIL_FROM,
                [requester.email],
                fail_silently=True,
                html_message=html_message,
            )

        return JsonResponse({"status": "success"}, status=200)
    else:
        return JsonResponse({"status": "invalid request"}, status=400)


def mark_as_failed(request, slug):
    if request.method == "GET" and get_client_ip(request) in ANSIBLE_SERVERS:
        application = Application.objects.get(slug=slug)
        for deploy in application.deployments.filter(status="Doing"):
            deploy.status = "Failed"
            deploy.save()
            if deploy.requester:
                html_message = loader.render_to_string(
                    "djangoldp_application/email.html",
                    {
                        "message": _("Deployment failed"),
                        "link": (getattr(settings, 'INSTANCE_DEFAULT_CLIENT', False) or getattr(settings, 'JABBER_DEFAULT_HOST')),
                        "object": "{} https://{}".format(
                            _("About your deployment of"), application.client_url
                        ),
                    },
                )

                send_mail(
                    "{} (https://{})".format(
                        _("Déploiement de ") + application.application_title,
                        application.client_url,
                    ),
                    _("Deployment failed"),
                    EMAIL_FROM,
                    [deploy.requester.email],
                    fail_silently=True,
                    html_message=html_message,
                )

        return JsonResponse({"status": "success"}, status=200)
    else:
        return JsonResponse({"status": "invalid request"}, status=400)



# Generated by Django 2.2.28 on 2023-03-31 06:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0004_auto_20230328_1657'),
    ]

    operations = [
        migrations.AddField(
            model_name='application',
            name='application_css_url',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='application',
            name='application_default_avatar_url',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='application',
            name='application_favicon_url',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='application',
            name='application_i18n_force',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='application',
            name='application_i18n_lang',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



import uuid

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.mail import send_mail
from django.db import models
from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver
from django.template import loader
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _
from djangoldp.models import Model
from djangoldp_component.models import Component, Package
from rest_framework.exceptions import ValidationError

EMAIL_FROM = getattr(settings, "DEFAULT_FROM_EMAIL", False) or getattr(
    settings, "EMAIL_HOST_USER", False
)


class Application(Model):
    repository = models.CharField(max_length=255, blank=True, null=True)
    friendly_name = models.CharField(max_length=255, blank=True, null=True)
    short_description = models.CharField(max_length=255, blank=True, null=True)
    creator = models.ForeignKey(
        get_user_model(),
        related_name="applications",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    api_url = models.CharField(
        max_length=255, blank=True, null=True, help_text='Without "http(s)://"'
    )
    client_url = models.CharField(
        max_length=255, blank=True, null=True, help_text='Without "http(s)://"'
    )
    # May move appliation_* to graphics?
    application_title = models.CharField(max_length=255, blank=True, null=True)
    application_logo = models.CharField(max_length=255, blank=True, null=True)
    slug = models.SlugField(unique=True, blank=True, null=True)

    @property
    def deploy(self):
        return self.deployments.filter(status="Todo").count() > 0

    def __str__(self):
        try:
            return "{} ({})".format(self.friendly_name, self.urlid)
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = ["add"]
        auto_author = "creator"
        container_path = "/applications/"
        depth = 2  # Do not serialize user
        lookup_field = "slug"
        nested_fields = ["components", "packages"]
        ordering = ["slug"]
        owner_field = "creator"
        owner_perms = ["view", "add", "change", "delete"]
        # rdf_context = {
        #     "friendly_name": "sib:friendlyName",
        #     "short_description": "sib:shortDescription",
        #     "creator": "foaf:user",
        # }
        rdf_type = "sib:application"
        serializer_fields = [
            "@id",
            "repository",
            "friendly_name",
            "short_description",
            "creator",
            "api_url",
            "client_url",
            "application_title",
            "application_logo",
            "components",
            "packages",
            "graphics",
            "services",
            "npms",
            "federation",
            "deployments",
        ]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("application")
        verbose_name_plural = _("applications")


STATUS_CHOICES = [
    ("Todo", _("Todo")),
    ("Doing", _("Doing")),
    ("Done", _("Done")),
    ("Failed", _("Failed")),
]


class Deployment(Model):
    application = models.ForeignKey(
        Application,
        on_delete=models.CASCADE,
        related_name="deployments",
        null=True,
        blank=True,
    )
    requester = models.ForeignKey(
        get_user_model(),
        related_name="deployments",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    date = models.DateTimeField(auto_now_add=True, verbose_name="Request date")
    resolutionDate = models.DateTimeField(auto_now=True, verbose_name="Resolution date")
    status = models.CharField(
        max_length=8, choices=STATUS_CHOICES, default="Todo", null=True, blank=True
    )

    def __str__(self):
        try:
            return "{} - {} ({})".format(
                self.date, self.application.friendly_name, self.application.urlid
            )
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = [
            "view",
            "add",
        ]  # workaround "view" permission, otherwise creator see other requester with only a "add" permission
        container_path = "/deployments/"
        depth = 0
        auto_author = "requester"
        owner_field = "requester"
        owner_perms = ["view", "add", "change", "delete"]
        ordering = ["application__urlid", "date"]
        # rdf_context = {
        #     "application": "sib:application",
        #     "date": "sib:deploymentDate",
        #     "resolutionDate": "sib:deploymentDate",
        #     "status": "sib:deploymentStatus",
        # }
        rdf_type = "sib:federation"
        serializer_fields = [
            "@id",
            "date",
            "resolutionDate",
            "status",
        ]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("deployment")
        verbose_name_plural = _("deployments")


class Federation(Model):
    application = models.ForeignKey(
        Application,
        on_delete=models.CASCADE,
        related_name="federation",
        null=True,
        blank=True,
    )
    target = models.ForeignKey(
        Application,
        on_delete=models.CASCADE,
        related_name="targeted_federation",
        null=True,
        blank=True,
    )

    def __str__(self):
        try:
            return "{} ({})".format(self.friendly_name, self.urlid)
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = ["add"]
        container_path = "/federations/"
        depth = 0
        # nested_fields = []
        ordering = ["application__urlid", "target__urlid"]
        owner_field = "application__creator"
        owner_perms = ["view", "add", "change", "delete"]
        # rdf_context = {
        #     "application": "sib:application",
        #     "target": "sib:application",
        # }
        rdf_type = "sib:federation"
        serializer_fields = [
            "@id",
            "target",
        ]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("federation")
        verbose_name_plural = _("federations")


class ApplicationGraphics(Model):
    application = models.ForeignKey(
        Application,
        on_delete=models.CASCADE,
        related_name="graphics",
        null=True,
        blank=True,
    )
    primary_key = models.CharField(max_length=255, blank=True, null=True)
    key = models.CharField(max_length=255, blank=True, null=True)
    value = models.TextField(blank=True, null=True)

    def __str__(self):
        try:
            return "{} + {}.{} ({})".format(
                self.application.friendly_name, self.primary_key, self.key, self.urlid
            )
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = ["add"]
        container_path = "application-graphics/"
        owner_field = "application__creator"
        owner_perms = ["view", "add", "change", "delete"]
        # rdf_context = {
        #     "application": "sib:application",
        #     "primary_key": "sib:key",
        #     "key": "sib:key",
        #     "value": "sib:value",
        # }
        rdf_type = "sib:graphic"
        serializer_fields = ["@id", "primary_key", "key", "value"]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("application graphic")
        verbose_name_plural = _("application graphics")


class ApplicationService(Model):
    application = models.ForeignKey(
        Application,
        on_delete=models.CASCADE,
        related_name="services",
        null=True,
        blank=True,
    )
    primary_key = models.CharField(max_length=255, blank=True, null=True)
    key = models.CharField(max_length=255, blank=True, null=True)
    value = models.TextField(blank=True, null=True)

    def __str__(self):
        try:
            return "{} + {}.{} ({})".format(
                self.application.friendly_name, self.primary_key, self.key, self.urlid
            )
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = ["add"]
        container_path = "application-services/"
        owner_field = "application__creator"
        owner_perms = ["view", "add", "change", "delete"]
        # rdf_context = {
        #     "application": "sib:application",
        #     "primary_key": "sib:key",
        #     "key": "sib:key",
        #     "value": "sib:value",
        # }
        rdf_type = "sib:service"
        serializer_fields = ["@id", "primary_key", "key", "value"]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("application service")
        verbose_name_plural = _("application services")


class ApplicationNPM(Model):
    application = models.ForeignKey(
        Application,
        on_delete=models.CASCADE,
        related_name="npms",
        null=True,
        blank=True,
    )
    package = models.CharField(max_length=255, blank=True, null=True)
    version = models.CharField(max_length=255, blank=True, null=True)

    def __str__(self):
        try:
            return "{} + {}@{} ({})".format(
                self.application.friendly_name, self.package, self.version, self.urlid
            )
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = ["add"]
        container_path = "application-npms/"
        owner_field = "application__creator"
        owner_perms = ["view", "add", "change", "delete"]
        # rdf_context = {
        #     "application": "sib:application",
        #     "package": "sib:npmpackage",
        #     "version": "sib:npmversion",
        # }
        rdf_type = "sib:npm"
        serializer_fields = ["@id", "package", "version"]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("application npm")
        verbose_name_plural = _("application npms")


class ApplicationComponent(Model):
    application = models.ForeignKey(
        Application,
        on_delete=models.CASCADE,
        related_name="components",
        null=True,
        blank=True,
    )
    component = models.ForeignKey(
        Component,
        on_delete=models.CASCADE,
        related_name="applications",
        null=True,
        blank=True,
    )

    def __str__(self):
        try:
            return "{} + {} ({})".format(
                self.application.friendly_name, self.component.friendly_name, self.urlid
            )
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = ["add"]
        container_path = "application-components/"
        nested_fields = ["component", "parameters"]
        owner_field = "application__creator"
        owner_perms = ["view", "add", "change", "delete"]
        # rdf_context = {
        #     "application": "sib:application",
        #     "component": "sib:component",
        # }
        rdf_type = "sib:dependency"
        serializer_fields = ["@id", "application", "component", "parameters"]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("application component")
        verbose_name_plural = _("application components")

    def save(self, *args, **kwargs):
        if (
            not self.pk
            and ApplicationComponent.objects.filter(
                component=self.component, application=self.application
            ).exists()
        ):
            return

        super(ApplicationComponent, self).save(*args, **kwargs)


class ApplicationComponentParameter(Model):
    component = models.ForeignKey(
        ApplicationComponent,
        on_delete=models.CASCADE,
        related_name="parameters",
        null=True,
        blank=True,
    )
    key = models.CharField(max_length=255, blank=True, null=True)
    value = models.CharField(max_length=255, blank=True, null=True)

    def __str__(self):
        try:
            return "{} -> {} ({})".format(
                self.component.application.friendly_name,
                self.component.component.friendly_name,
                self.urlid,
            )
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = ["add"]
        container_path = "application-component-parameters/"
        owner_field = "component__application__creator"
        owner_perms = ["view", "add", "change", "delete"]
        # rdf_context = {
        #     "component": "sib:dependency",
        #     "key": "sib:key",
        #     "value": "sib:value",
        # }
        rdf_type = "sib:parameter"
        serializer_fields = ["@id", "key", "value"]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("component parameter")
        verbose_name_plural = _("component parameters")


class ApplicationPackage(Model):
    application = models.ForeignKey(
        Application,
        on_delete=models.CASCADE,
        related_name="packages",
        null=True,
        blank=True,
    )
    package = models.ForeignKey(
        Package,
        on_delete=models.CASCADE,
        related_name="applications",
        null=True,
        blank=True,
    )

    def __str__(self):
        try:
            return "{} + {} ({})".format(
                self.application.friendly_name, self.package.friendly_name, self.urlid
            )
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = ["add"]
        container_path = "application-packages/"
        nested_fields = ["package", "parameters"]
        owner_field = "application__creator"
        owner_perms = ["view", "add", "change", "delete"]
        # rdf_context = {
        #     "application": "sib:application",
        #     "package": "sib:package",
        # }
        rdf_type = "sib:dependency"
        serializer_fields = ["@id", "application", "package", "parameters"]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("application package")
        verbose_name_plural = _("application packages")

    def save(self, *args, **kwargs):
        if (
            not self.pk
            and ApplicationPackage.objects.filter(
                package=self.package, application=self.application
            ).exists()
        ):
            return

        super(ApplicationPackage, self).save(*args, **kwargs)


class ApplicationPackageParameter(Model):
    package = models.ForeignKey(
        ApplicationPackage,
        on_delete=models.CASCADE,
        related_name="parameters",
        null=True,
        blank=True,
    )
    key = models.CharField(max_length=255, blank=True, null=True)
    value = models.CharField(max_length=255, blank=True, null=True)

    def __str__(self):
        try:
            return "{} -> {} ({})".format(
                self.package.application.friendly_name,
                self.package.package.friendly_name,
                self.urlid,
            )
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = ["add"]
        container_path = "application-package-parameters/"
        owner_field = "package__application__creator"
        owner_perms = ["view", "add", "change", "delete"]
        # rdf_context = {
        #     "package": "sib:dependency",
        #     "key": "sib:key",
        #     "value": "sib:value",
        # }
        rdf_type = "sib:parameter"
        serializer_fields = ["@id", "key", "value"]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("package parameter")
        verbose_name_plural = _("package parameters")


class ComponentExtension(Model):
    base_component = models.ForeignKey(
        ApplicationComponent,
        on_delete=models.CASCADE,
        related_name="extensions",
        null=True,
        blank=True,
    )
    component = models.ForeignKey(
        Component,
        on_delete=models.CASCADE,
        related_name="extends",
        null=True,
        blank=True,
    )

    def __str__(self):
        try:
            return "{} + {} ({})".format(
                self.base_component.application.friendly_name,
                self.component.friendly_name,
                self.urlid,
            )
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = ["add"]
        container_path = "component-extensions/"
        nested_fields = ["component", "base_component"]
        owner_field = "base_component__application__creator"
        owner_perms = ["view", "add", "change", "delete"]
        # rdf_context = {
        #     "base_component": "sib:dependency",
        #     "component": "sib:component",
        # }
        rdf_type = "sib:dependency"
        serializer_fields = ["@id", "component"]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("component extension")
        verbose_name_plural = _("component extensions")

    def save(self, *args, **kwargs):
        if (
            not self.pk
            and ComponentExtension.objects.filter(
                base_component=self.base_component, component=self.component
            ).exists()
        ):
            return

        super(ComponentExtension, self).save(*args, **kwargs)


class ComponentExtensionParameter(Model):
    component = models.ForeignKey(
        ComponentExtension,
        on_delete=models.CASCADE,
        related_name="parameters",
        null=True,
        blank=True,
    )
    key = models.CharField(max_length=255, blank=True, null=True)
    value = models.CharField(max_length=255, blank=True, null=True)

    def __str__(self):
        try:
            return "{} -> {} ({})".format(
                self.component.base_component.application.friendly_name,
                self.component.component.friendly_name,
                self.urlid,
            )
        except:
            return self.urlid

    class Meta(Model.Meta):
        anonymous_perms = []
        authenticated_perms = ["add"]
        container_path = "component-extension-parameters/"
        owner_field = "component__base_component__application__creator"
        owner_perms = ["view", "add", "change", "delete"]
        # rdf_context = {
        #     "component": "sib:dependency",
        #     "key": "sib:key",
        #     "value": "sib:value",
        # }
        rdf_type = "sib:parameter"
        serializer_fields = ["@id", "key", "value"]
        superuser_perms = ["view", "add", "change", "delete"]
        verbose_name = _("component extension parameter")
        verbose_name_plural = _("component extension parameters")


@receiver(pre_save, sender=Application)
def pre_save_slugify(sender, instance, **kwargs):
    if not instance.urlid or instance.urlid.startswith(settings.SITE_URL):
        if getattr(instance, Model.slug_field(instance)) != slugify(
            instance.friendly_name
        ):
            if (
                sender.objects.local()
                .filter(slug=slugify(instance.friendly_name))
                .count()
                > 0
            ):
                raise ValidationError(sender.__name__ + str(_(" must be unique")))
            setattr(
                instance, Model.slug_field(instance), slugify(instance.friendly_name)
            )
            setattr(instance, "urlid", "")
    else:
        # Is a distant object, generate a random slug
        setattr(instance, Model.slug_field(instance), uuid.uuid4().hex.upper()[0:8])

@receiver(post_save, sender=Application)
def post_create_mail_admins(created, instance, **kwargs):
    if created:
        if instance.client_url.endswith(".otc.startinblox.com"):
            html_message = loader.render_to_string(
                "djangoldp_application/email.html",
                {
                    "message": "{} {}\n{} {}".format(
                        _("A new application have been created:"),
                        instance.client_url,
                        _("Created by:"),
                        instance.creator.get_full_name(),
                    ),
                    "link": (getattr(settings, 'INSTANCE_DEFAULT_CLIENT', False) or getattr(settings, 'JABBER_DEFAULT_HOST')),
                    "object": "{} (https://{})".format(
                        _("A new application have been created"),
                        instance.client_url
                    ),
                },
            )

            send_mail(
                "{} (https://{})".format(
                    _("A new application have been created ") + instance.application_title,
                    instance.client_url,
                ),
                "{} (https://{})".format(
                    _("A new application have been created ") + instance.application_title,
                    instance.client_url,
                ),
                EMAIL_FROM,
                get_user_model().objects.filter(is_superuser=True).values_list('email'),
                fail_silently=True,
                html_message=html_message,
            )



# Generated by Django 2.2.28 on 2023-05-12 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0013_auto_20230512_1608'),
    ]

    operations = [
        migrations.AlterField(
            model_name='application',
            name='api_url',
            field=models.CharField(blank=True, help_text='Without "http(s)://"', max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='application',
            name='client_url',
            field=models.CharField(blank=True, help_text='Without "http(s)://"', max_length=255, null=True),
        ),
    ]



# Generated by Django 2.2.28 on 2023-05-12 13:55

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('djangoldp_application', '0010_auto_20230331_0921'),
    ]

    operations = [
        migrations.AddField(
            model_name='deployment',
            name='requester',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='deployments', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='deployment',
            name='resolutionDate',
            field=models.DateTimeField(auto_now=True),
        ),
    ]



from django import forms
from django.contrib import admin
from django.core.exceptions import ValidationError
from djangoldp.admin import DjangoLDPAdmin

from .models import (
    Application,
    ApplicationComponent,
    ApplicationComponentParameter,
    ApplicationGraphics,
    ApplicationNPM,
    ApplicationPackage,
    ApplicationPackageParameter,
    ApplicationService,
    ComponentExtension,
    ComponentExtensionParameter,
    Deployment,
    Federation,
)


@admin.register(ApplicationComponentParameter, ApplicationGraphics, ApplicationNPM, ApplicationPackageParameter, ApplicationService, ComponentExtensionParameter, Federation)
class EmptyAdmin(admin.ModelAdmin):
    def get_model_perms(self, request):
        return {}


class ApplicationComponentInline(admin.TabularInline):
    model = ApplicationComponent
    exclude = ("urlid", "is_backlink", "allow_create_backlink")
    fields = ("component",)
    show_change_link = True
    extra = 0


class ApplicationServiceInline(admin.TabularInline):
    model = ApplicationService
    exclude = ("urlid", "is_backlink", "allow_create_backlink")
    fields = ("primary_key", "key", "value")
    extra = 0


class ApplicationGraphicsInline(admin.TabularInline):
    model = ApplicationGraphics
    exclude = ("urlid", "is_backlink", "allow_create_backlink")
    fields = ("primary_key", "key", "value")
    extra = 0


class ApplicationNPMInline(admin.TabularInline):
    model = ApplicationNPM
    exclude = ("urlid", "is_backlink", "allow_create_backlink")
    fields = ("package", "version")
    extra = 0


class ApplicationPackageInline(admin.TabularInline):
    model = ApplicationPackage
    exclude = ("urlid", "is_backlink", "allow_create_backlink")
    fields = ("package",)
    show_change_link = True
    extra = 0


class FederationInline(admin.TabularInline):
    model = Federation
    fk_name = "application"
    exclude = ("urlid", "is_backlink", "allow_create_backlink")
    extra = 0


@admin.register(Application)
class ApplicationAdmin(DjangoLDPAdmin):
    list_display = ("urlid", "friendly_name", "short_description")
    exclude = (
        "urlid",
        "slug",
        "is_backlink",
        "allow_create_backlink",
    )
    inlines = [
        ApplicationGraphicsInline,
        ApplicationServiceInline,
        ApplicationNPMInline,
        ApplicationComponentInline,
        ApplicationPackageInline,
        FederationInline,
    ]
    search_fields = ["urlid", "friendly_name", "short_description"]
    ordering = ["slug"]


class ApplicationPackageParameterInline(admin.TabularInline):
    model = ApplicationPackageParameter
    exclude = ("urlid", "is_backlink", "allow_create_backlink")
    extra = 0


@admin.register(ApplicationPackage)
class ApplicationPackageAdmin(DjangoLDPAdmin):
    list_display = ("urlid", "application", "package")
    exclude = ("urlid", "slug", "is_backlink", "allow_create_backlink")
    inlines = [ApplicationPackageParameterInline]
    search_fields = [
        "urlid",
        "package__friendly_name",
        "package__short_description",
        "application__friendly_name",
        "application__short_description",
    ]
    ordering = ["application__slug", "package__slug"]


class ApplicationComponentParameterInline(admin.TabularInline):
    model = ApplicationComponentParameter
    exclude = ("urlid", "is_backlink", "allow_create_backlink")
    extra = 0


@admin.register(ApplicationComponent)
class ApplicationComponentAdmin(DjangoLDPAdmin):
    list_display = ("urlid", "application", "component")
    exclude = ("urlid", "slug", "is_backlink", "allow_create_backlink")
    inlines = [ApplicationComponentParameterInline]
    search_fields = [
        "urlid",
        "component__friendly_name",
        "component__short_description",
        "application__friendly_name",
        "application__short_description",
    ]
    ordering = ["application__slug", "component__slug"]


class ComponentExtensionParameterInline(admin.TabularInline):
    model = ComponentExtensionParameter
    exclude = ("urlid", "is_backlink", "allow_create_backlink")
    extra = 0


@admin.register(ComponentExtension)
class ComponentExtensionAdmin(DjangoLDPAdmin):
    list_display = ("urlid", "base_component", "component")
    exclude = ("urlid", "slug", "is_backlink", "allow_create_backlink")
    inlines = [ComponentExtensionParameterInline]
    search_fields = [
        "urlid",
        "component__friendly_name",
        "component__short_description",
        "base_component__component__friendly_name",
        "base_component__component__short_description",
        "base_component__application__friendly_name",
        "base_component__application__short_description",
    ]
    ordering = ["base_component__application__slug", "component__slug"]


@admin.register(Deployment)
class DeploymentAdmin(DjangoLDPAdmin):
    list_display = ("application", "requester", "date", "status", "resolutionDate")
    exclude = ("urlid", "slug", "is_backlink", "allow_create_backlink")
    search_fields = [
        "urlid",
        "application__slug",
        "requester__name",
        "application__friendly_name",
    ]
    ordering = ["application__slug", "date", "resolutionDate"]





# Generated by Django 2.2.28 on 2023-03-17 13:28

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import djangoldp.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('djangoldp_component', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ApplicationTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('friendly_name', models.CharField(blank=True, max_length=255, null=True)),
                ('short_description', models.CharField(blank=True, max_length=255, null=True)),
                ('slug', models.SlugField(blank=True, null=True, unique=True)),
                ('creator', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='templates', to=settings.AUTH_USER_MODEL)),
                ('repository', models.CharField(blank=True, max_length=255, null=True)),
            ],
            options={
                'verbose_name': 'template',
                'verbose_name_plural': 'templates',
                'ordering': ['slug'],
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'lookup_field': 'slug',
                'rdf_type': 'sib:template',
                'rdf_context': {'creator': 'foaf:user', 'friendly_name': 'sib:friendlyName', 'short_description': 'sib:shortDescription'},
                'auto_author': 'creator',
                'owner_field': 'creator',
                'container_path': '/templates/',
                'serializer_fields': ['@id', 'friendly_name', 'short_description', 'creator', 'components', 'packages'],
                'nested_fields': ['components', 'packages'],
                'depth': 1,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit', 'add'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['view'],
            },
        ),
        migrations.CreateModel(
            name='Application',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('friendly_name', models.CharField(blank=True, max_length=255, null=True)),
                ('short_description', models.CharField(blank=True, max_length=255, null=True)),
                ('slug', models.SlugField(blank=True, null=True, unique=True)),
                ('creator', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='applications', to=settings.AUTH_USER_MODEL)),
                ('application_logo', models.CharField(blank=True, max_length=255, null=True)),
                ('application_title', models.CharField(blank=True, max_length=255, null=True)),
                ('client_url', models.CharField(blank=True, max_length=255, null=True)),
                ('template', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='applications', to='djangoldp_application.ApplicationTemplate')),
            ],
            options={
                'verbose_name': 'application',
                'verbose_name_plural': 'applications',
                'ordering': ['slug'],
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'lookup_field': 'slug',
                'rdf_type': 'sib:application',
                'rdf_context': {'creator': 'foaf:user', 'friendly_name': 'sib:friendlyName', 'short_description': 'sib:shortDescription'},
                'auto_author': 'creator',
                'owner_field': 'creator',
                'container_path': '/applications/',
                'serializer_fields': ['@id', 'friendly_name', 'short_description', 'creator'],
                'depth': 1,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit', 'add'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['view'],
            },
        ),
        migrations.CreateModel(
            name='TemplateRequiredPackage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('package', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='templates', to='djangoldp_component.Package')),
                ('template', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='packages', to='djangoldp_application.ApplicationTemplate')),
            ],
            options={
                'verbose_name': 'template required package',
                'verbose_name_plural': 'template required packages',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:dependency',
                'rdf_context': {'package': 'sib:package', 'template': 'sib:template'},
                'owner_field': 'template__creator',
                'container_path': 'template-required-packages/',
                'serializer_fields': ['@id', 'package'],
                'nested_fields': ['package'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['inherit'],
            },
        ),
        migrations.CreateModel(
            name='TemplateRequiredComponent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('component', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='templates', to='djangoldp_component.Component')),
                ('template', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='components', to='djangoldp_application.ApplicationTemplate')),
            ],
            options={
                'verbose_name': 'template required component',
                'verbose_name_plural': 'template required components',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:dependency',
                'rdf_context': {'component': 'sib:component', 'template': 'sib:template'},
                'owner_field': 'template__creator',
                'container_path': 'template-required-components/',
                'serializer_fields': ['@id', 'component'],
                'nested_fields': ['component'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['inherit'],
            },
        ),
        migrations.CreateModel(
            name='Federation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('application', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='federation', to='djangoldp_application.Application')),
                ('target', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='targeted_federation', to='djangoldp_application.Application')),
            ],
            options={
                'verbose_name': 'federation',
                'verbose_name_plural': 'federations',
                'ordering': ['application__urlid', 'target__urlid'],
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:federation',
                'rdf_context': {'application': 'sib:application', 'target': 'sib:application'},
                'owner_field': 'application__creator',
                'container_path': '/federations/',
                'serializer_fields': ['@id', 'target'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit', 'add'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['view'],
            },
        ),
        migrations.CreateModel(
            name='ApplicationComponent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('application', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='components', to='djangoldp_application.Application')),
                ('component', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='applications', to='djangoldp_component.Component')),
            ],
            options={
                'verbose_name': 'application component',
                'verbose_name_plural': 'application components',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:dependency',
                'rdf_context': {'application': 'sib:application', 'component': 'sib:component'},
                'owner_field': 'application__creator',
                'container_path': 'application-components/',
                'serializer_fields': ['@id', 'component'],
                'nested_fields': ['component'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['inherit'],
            },
        ),
        migrations.CreateModel(
            name='ApplicationPackage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('application', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='packages', to='djangoldp_application.Application')),
                ('package', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='applications', to='djangoldp_component.Package')),
            ],
            options={
                'verbose_name': 'application package',
                'verbose_name_plural': 'application packages',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:dependency',
                'rdf_context': {'application': 'sib:application', 'package': 'sib:package'},
                'owner_field': 'application__creator',
                'container_path': 'application-packages/',
                'serializer_fields': ['@id', 'package', 'parameters'],
                'nested_fields': ['package', 'parameters'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['inherit'],
            },
        ),
        migrations.CreateModel(
            name='ApplicationPackageParameter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('key', models.CharField(blank=True, max_length=255, null=True)),
                ('value', models.CharField(blank=True, max_length=255, null=True)),
                ('package', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='parameters', to='djangoldp_application.ApplicationPackage')),
            ],
            options={
                'verbose_name': 'package parameter',
                'verbose_name_plural': 'package parameters',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:parameter',
                'rdf_context': {'key': 'sib:key', 'package': 'sib:dependency', 'value': 'sib:value'},
                'owner_field': 'package__application__creator',
                'container_path': 'application-package-parameters/',
                'serializer_fields': ['@id', 'key', 'value'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['inherit'],
            },
        ),
        migrations.CreateModel(
            name='ApplicationComponentParameter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('key', models.CharField(blank=True, max_length=255, null=True)),
                ('value', models.CharField(blank=True, max_length=255, null=True)),
                ('component', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='parameters', to='djangoldp_application.ApplicationComponent')),
            ],
            options={
                'verbose_name': 'component parameter',
                'verbose_name_plural': 'component parameters',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:parameter',
                'rdf_context': {'component': 'sib:dependency', 'key': 'sib:key', 'value': 'sib:value'},
                'owner_field': 'component__application__creator',
                'container_path': 'application-component-parameters/',
                'serializer_fields': ['@id', 'key', 'value'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['inherit'],
            },
        ),
        migrations.CreateModel(
            name='ComponentExtension',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('base_component', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='extensions', to='djangoldp_application.ApplicationComponent')),
                ('component', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='extends', to='djangoldp_component.Component')),
            ],
            options={
                'verbose_name': 'component extension',
                'verbose_name_plural': 'component extensions',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:dependency',
                'rdf_context': {'base_component': 'sib:dependency', 'component': 'sib:component'},
                'owner_field': 'base_component__application__creator',
                'container_path': 'component-extensions/',
                'serializer_fields': ['@id', 'component'],
                'nested_fields': ['component', 'base_component'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['inherit'],
            },
        ),
        migrations.CreateModel(
            name='ComponentExtensionParameter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('key', models.CharField(blank=True, max_length=255, null=True)),
                ('value', models.CharField(blank=True, max_length=255, null=True)),
                ('component', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='parameters', to='djangoldp_application.ComponentExtension')),
            ],
            options={
                'verbose_name': 'component extension parameter',
                'verbose_name_plural': 'component extension parameters',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:parameter',
                'rdf_context': {'component': 'sib:dependency', 'key': 'sib:key', 'value': 'sib:value'},
                'owner_field': 'component__base_component__application__creator',
                'container_path': 'component-extension-parameters/',
                'serializer_fields': ['@id', 'key', 'value'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['inherit'],
            },
        ),
    ]



# Generated by Django 2.2.28 on 2023-05-12 13:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0011_auto_20230512_1555'),
    ]

    operations = [
        migrations.AlterField(
            model_name='deployment',
            name='status',
            field=models.CharField(blank=True, choices=[('Todo', 'Todo'), ('Doing', 'Doing'), ('Done', 'Done'), ('Failed', 'Failed')], default='Todo', max_length=8, null=True),
        ),
    ]



# Generated by Django 2.2.28 on 2023-03-28 14:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0003_application_api_url'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='templaterequiredcomponent',
            name='component',
        ),
        migrations.RemoveField(
            model_name='templaterequiredcomponent',
            name='template',
        ),
        migrations.RemoveField(
            model_name='templaterequiredpackage',
            name='package',
        ),
        migrations.RemoveField(
            model_name='templaterequiredpackage',
            name='template',
        ),
        migrations.RemoveField(
            model_name='application',
            name='template',
        ),
        migrations.AddField(
            model_name='application',
            name='repository',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.DeleteModel(
            name='ApplicationTemplate',
        ),
        migrations.DeleteModel(
            name='TemplateRequiredComponent',
        ),
        migrations.DeleteModel(
            name='TemplateRequiredPackage',
        ),
    ]



from django.urls import path

from .views import (
    ApplicationDetailViewSet,
    ApplicationViewSet,
    CreateApplicationViewSet,
    mark_as_doing,
    mark_as_done,
    mark_as_failed,
)

urlpatterns = [
    path(
        "create-application/",
        CreateApplicationViewSet.as_view({"get": "list", "post": "create"}),
        name="create_application",
    ),
    path(
        "ansible/inventory/",
        ApplicationViewSet.as_view({"get": "list"}),
        name="ansible_inventory",
    ),
    path(
        "ansible/<slug:slug>/",
        ApplicationDetailViewSet.as_view({"get": "retrieve"}, lookup_field="slug"),
        name="ansible_inventory_detail",
    ),
    path(
        "ansible/<slug:slug>/doing/",
        mark_as_doing,
        name="ansible_inventory_mark_doing",
    ),
    path(
        "ansible/<slug:slug>/done/",
        mark_as_done,
        name="ansible_inventory_mark_done",
    ),
    path(
        "ansible/<slug:slug>/failed/",
        mark_as_failed,
        name="ansible_inventory_mark_failed",
    ),
]



# Generated by Django 2.2.28 on 2023-03-31 06:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0006_applicationservice'),
    ]

    operations = [
        migrations.AlterField(
            model_name='applicationservice',
            name='value',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.28 on 2023-03-31 07:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0009_applicationgraphics'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='application',
            name='application_css_url',
        ),
        migrations.RemoveField(
            model_name='application',
            name='application_default_avatar_url',
        ),
        migrations.RemoveField(
            model_name='application',
            name='application_favicon_url',
        ),
        migrations.RemoveField(
            model_name='application',
            name='application_i18n_force',
        ),
        migrations.RemoveField(
            model_name='application',
            name='application_i18n_lang',
        ),
    ]



# Generated by Django 2.2.28 on 2023-05-12 14:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0012_auto_20230512_1556'),
    ]

    operations = [
        migrations.AlterField(
            model_name='deployment',
            name='date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Request date'),
        ),
        migrations.AlterField(
            model_name='deployment',
            name='resolutionDate',
            field=models.DateTimeField(auto_now=True, verbose_name='Resolution date'),
        ),
    ]



# Generated by Django 2.2.28 on 2023-03-31 07:10

from django.db import migrations, models
import django.db.models.deletion
import djangoldp.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0007_auto_20230331_0857'),
    ]

    operations = [
        migrations.CreateModel(
            name='ApplicationNPM',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('package', models.CharField(blank=True, max_length=255, null=True)),
                ('version', models.CharField(blank=True, max_length=255, null=True)),
                ('application', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='npms', to='djangoldp_application.Application')),
            ],
            options={
                'verbose_name': 'application npm',
                'verbose_name_plural': 'application npms',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:npm',
                'rdf_context': {'application': 'sib:application', 'package': 'sib:npmpackage', 'version': 'sib:npmversion'},
                'owner_field': 'application__creator',
                'container_path': 'application-npms/',
                'serializer_fields': ['@id', 'package', 'version'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['inherit'],
            },
        ),
    ]
